@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v0.1.3
+## Version v0.1.4
 ## Copyright(c) 2020 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <@sitael.tokyo.deguest.jp>
 ## Created 2020/05/17
-## Modified 2020/05/21
+## Modified 2020/05/25
 ## 
 ##----------------------------------------------------------------------------
 package Nice::Try;
@@ -16,8 +16,9 @@ BEGIN
     use PPI;
     use Filter::Util::Call;
     use Scalar::Util;
+    use List::Util ();
     # use Devel::Confess;
-    our $VERSION = 'v0.1.3';
+    our $VERSION = 'v0.1.4';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -377,7 +378,8 @@ sub _parse
         if( scalar( @$fin_block_ref ) )
         {
             my $fin_def = $fin_block_ref->[0];
-            my $finally_block = $fin_def->{block}->content;
+            ## my $finally_block = $fin_def->{block}->content;
+            my $finally_block = $self->_serialize( $fin_def->{block} );
             $finally_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
             $fin_block = <<EOT;
 CORE::local \$Nice::Try::FINALLY = Nice\::Try\::ScopeGuard->_new(sub __FINALLY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ __FINALLY__CLOSE_NL__}, \@_);
@@ -423,7 +425,8 @@ EOT
                 $try_def->{block} = $emb;
             }
                         
-            my $try_block = $try_def->{block}->content;
+            ## my $try_block = $try_def->{block}->content;
+            my $try_block = $self->_serialize( $try_def->{block} );
             $try_block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
             
             my $try_sub = <<EOT;
@@ -522,7 +525,8 @@ EOT
                 if( $cdef->{var} )
                 {
                     ## $self->_messagef( 3, "Catch assignment is: '%s'", $cdef->{var}->content );
-                    my $str = $cdef->{var}->content;
+                    ## my $str = $cdef->{var}->content;
+                    my $str = $self->_serialize( $cdef->{var} );
                     $str =~ s/^\([[:blank:]]*|[[:blank:]]*\)$//g;
                     if( $str =~ /^(\S+)[[:blank:]]+(\$\S+)$/ )
                     {
@@ -566,7 +570,8 @@ EOT
                     $cond = 'elsif';
                 }
                 ## $self->_message( 3, "\$i = $i, \$total_catch = $total_catch and cond = '$cond'" );
-                my $block = $cdef->{block}->content;
+                ## my $block = $cdef->{block}->content;
+                my $block = $self->_serialize( $cdef->{block} );
                 $block =~ s/^\{[[:blank:]]*|[[:blank:]]*\}$//gs;
                 my $catch_section = '';
                 my $catch_code = <<EOT;
@@ -610,7 +615,8 @@ EOT
                         ## $self->_message( 3, "Called here for fallback for element No $i" );
                         if( ++$else > 1 )
                         {
-                            CORE::warn( "Cannot have more than one falllback catch clause for block: ", $cdef->{block}->content, "\n" ) if( warnings::enabled );
+                            ## CORE::warn( "Cannot have more than one falllback catch clause for block: ", $cdef->{block}->content, "\n" ) if( warnings::enabled );
+                            CORE::warn( "Cannot have more than one falllback catch clause for block: ", $self->_serialize( $cdef->{block} ), "\n" ) if( warnings::enabled );
                             ## Skip, not die. Not fatal, just ignored
                             next;
                         }
@@ -738,6 +744,139 @@ EOT
     return( $elem );
 }
 
+## Taken from PPI::Document
+sub _serialize 
+{
+    my $self   = shift( @_ );
+	my $ppi    = shift( @_ ) || return( '' );
+	my @tokens = $ppi->tokens;
+
+	# The here-doc content buffer
+	my $heredoc = '';
+
+	# Start the main loop
+	my $output = '';
+	foreach my $i ( 0 .. $#tokens ) {
+		my $Token = $tokens[$i];
+
+		# Handle normal tokens
+		unless ( $Token->isa('PPI::Token::HereDoc') ) {
+			my $content = $Token->content;
+
+			# Handle the trivial cases
+			unless ( $heredoc ne '' and $content =~ /\n/ ) {
+				$output .= $content;
+				next;
+			}
+
+			# We have pending here-doc content that needs to be
+			# inserted just after the first newline in the content.
+			if ( $content eq "\n" ) {
+				# Shortcut the most common case for speed
+				$output .= $content . $heredoc;
+			} else {
+				# Slower and more general version
+				$content =~ s/\n/\n$heredoc/;
+				$output .= $content;
+			}
+
+			$heredoc = '';
+			next;
+		}
+
+		# This token is a HereDoc.
+		# First, add the token content as normal, which in this
+		# case will definitely not contain a newline.
+		$output .= $Token->content;
+
+		# Now add all of the here-doc content to the heredoc buffer.
+		foreach my $line ( $Token->heredoc ) {
+			$heredoc .= $line;
+		}
+
+		if ( $Token->{_damaged} ) {
+			# Special Case:
+			# There are a couple of warning/bug situations
+			# that can occur when a HereDoc content was read in
+			# from the end of a file that we silently allow.
+			#
+			# When writing back out to the file we have to
+			# auto-repair these problems if we aren't going back
+			# on to the end of the file.
+
+			# When calculating $last_line, ignore the final token if
+			# and only if it has a single newline at the end.
+			my $last_index = $#tokens;
+			if ( $tokens[$last_index]->{content} =~ /^[^\n]*\n$/ ) {
+				$last_index--;
+			}
+
+			# This is a two part test.
+			# First, are we on the last line of the
+			# content part of the file
+			my $last_line = List::Util::none {
+				$tokens[$_] and $tokens[$_]->{content} =~ /\n/
+				} (($i + 1) .. $last_index);
+			if ( ! defined $last_line ) {
+				# Handles the null list case
+				$last_line = 1;
+			}
+
+			# Secondly, are their any more here-docs after us,
+			# (with content or a terminator)
+			my $any_after = List::Util::any {
+				$tokens[$_]->isa('PPI::Token::HereDoc')
+				and (
+					scalar(@{$tokens[$_]->{_heredoc}})
+					or
+					defined $tokens[$_]->{_terminator_line}
+					)
+				} (($i + 1) .. $#tokens);
+			if ( ! defined $any_after ) {
+				# Handles the null list case
+				$any_after = '';
+			}
+
+			# We don't need to repair the last here-doc on the
+			# last line. But we do need to repair anything else.
+			unless ( $last_line and ! $any_after ) {
+				# Add a terminating string if it didn't have one
+				unless ( defined $Token->{_terminator_line} ) {
+					$Token->{_terminator_line} = $Token->{_terminator};
+				}
+
+				# Add a trailing newline to the terminating
+				# string if it didn't have one.
+				unless ( $Token->{_terminator_line} =~ /\n$/ ) {
+					$Token->{_terminator_line} .= "\n";
+				}
+			}
+		}
+
+		# Now add the termination line to the heredoc buffer
+		if ( defined $Token->{_terminator_line} ) {
+			$heredoc .= $Token->{_terminator_line};
+		}
+	}
+
+	# End of tokens
+
+	if ( $heredoc ne '' ) {
+		# If the file doesn't end in a newline, we need to add one
+		# so that the here-doc content starts on the next line.
+		unless ( $output =~ /\n$/ ) {
+			$output .= "\n";
+		}
+
+		# Now we add the remaining here-doc content
+		# to the end of the file.
+		$output .= $heredoc;
+	}
+
+	$output;
+}
+
+
 {
   package # hide from PAUSE
     Nice::Try::ScopeGuard;
@@ -831,7 +970,7 @@ When run, this would produce, as one would expect:
 
 =head1 VERSION
 
-    v0.1.3
+    v0.1.4
 
 =head1 DESCRIPTION
 
